package com.example.mobileapp

import android.app.Application
import androidx.room.*
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.repositories.RestCategoryRepository
import com.example.mobileapp.data.api.repositories.RestOrderProductRepository
import com.example.mobileapp.data.api.repositories.RestOrderRepository
import com.example.mobileapp.data.api.repositories.RestProductRepository
import com.example.mobileapp.data.api.repositories.RestRoleRepository
import com.example.mobileapp.data.api.repositories.RestStatusRepository
import com.example.mobileapp.data.api.repositories.RestUserRepository
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.repositories.CategoryRepository
import com.example.mobileapp.data.repositories.OrderProductRepository
import com.example.mobileapp.data.repositories.OrderRepository
import com.example.mobileapp.data.repositories.ProductRepository
import com.example.mobileapp.data.repositories.RoleRepository
import com.example.mobileapp.data.repositories.StatusRepository
import com.example.mobileapp.data.repositories.UserRepository
import dagger.Module
import dagger.Provides
import dagger.hilt.InstallIn
import dagger.hilt.components.SingletonComponent
import javax.inject.Singleton

@Module
@InstallIn(SingletonComponent::class)
object AppModule {
    @Provides
    @Singleton
    fun provideDatabase(app: Application) : AppDatabase {
        return Room.databaseBuilder(
            app,
            AppDatabase::class.java,
            AppDatabase.DB_NAME
        )
            .fallbackToDestructiveMigration()
            .build()
    }

    @Provides
    @Singleton
    fun provideServerService() : MyServerService {
        return MyServerService.getInstance()
    }

    @Provides
    @Singleton
    fun provideRestUserRepository(service: MyServerService) : RestUserRepository {
        return RestUserRepository(service)
    }

    @Provides
    @Singleton
    fun provideCategoryRepository(db: AppDatabase, restCategoryRepository: RestCategoryRepository) : CategoryRepository {
        return CategoryRepository(db.categoryDao(), db, restCategoryRepository)
    }

    @Provides
    @Singleton
    fun provideOrderProductRepository(db: AppDatabase, restOrderProductRepository: RestOrderProductRepository) : OrderProductRepository {
        return OrderProductRepository(db.orderProductDao(), restOrderProductRepository)
    }

    @Provides
    @Singleton
    fun provideOrderRepository(db: AppDatabase, restOrderRepository: RestOrderRepository) : OrderRepository {
        return OrderRepository(db, db.orderDao(), db.orderProductDao(), restOrderRepository)
    }

    @Provides
    @Singleton
    fun provideProductRepository(db: AppDatabase, restProductRepository: RestProductRepository) : ProductRepository {
        return ProductRepository(db.productDao(), db.orderProductDao(), restProductRepository)
    }

    @Provides
    @Singleton
    fun provideRoleRepository(db: AppDatabase, restRoleRepository: RestRoleRepository) : RoleRepository {
        return RoleRepository(db.roleDao(), restRoleRepository)
    }

    @Provides
    @Singleton
    fun provideStatusRepository(db: AppDatabase, restStatusRepository: RestStatusRepository) : StatusRepository {
        return StatusRepository(db.statusDao(), restStatusRepository)
    }

    @Provides
    @Singleton
    fun provideUserRepository(db: AppDatabase, restUserRepository: RestUserRepository) : UserRepository {
        return UserRepository(db.userDao(), db, restUserRepository)
    }
}