package com.example.mobileapp.data.api

import com.example.mobileapp.data.api.models.CategoryListResponse
import com.example.mobileapp.data.api.models.CategoryRemote
import com.example.mobileapp.data.api.models.OrderListResponse
import com.example.mobileapp.data.api.models.OrderProductRemote
import com.example.mobileapp.data.api.models.OrderRemote
import com.example.mobileapp.data.api.models.ProductRemote
import com.example.mobileapp.data.api.models.RoleRemote
import com.example.mobileapp.data.api.models.StatusRemote
import com.example.mobileapp.data.api.models.TokenRemote
import com.example.mobileapp.data.api.models.UserListResponse
import com.example.mobileapp.data.api.models.UserRemote
import com.jakewharton.retrofit2.converter.kotlinx.serialization.asConverterFactory
import kotlinx.serialization.json.Json
import okhttp3.MediaType.Companion.toMediaType
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.http.Body
import retrofit2.http.DELETE
import retrofit2.http.GET
import retrofit2.http.Header
import retrofit2.http.POST
import retrofit2.http.PUT
import retrofit2.http.Path

interface MyServerService {
    // User
    @POST("register")
    suspend fun createUser(
        @Body user: UserRemote,
    ): UserRemote

    @POST("auth")
    suspend fun getToken(
        @Body user: UserRemote
    ) : TokenRemote

    @POST("check/login")
    suspend fun checkLogin(
        @Body user: UserRemote
    ) : UserRemote?

    @GET("users/{after}")
    suspend fun getUsersList(
        @Path("after") after: Int,
        @Header("Authorization") token: String
    ) : UserListResponse

    @PUT("user/update")
    suspend fun updateUser(
        @Body user: UserRemote,
        @Header("Authorization") token: String
    ): UserRemote

    @DELETE("user/delete/{id}")
    suspend fun deleteUser(
        @Path("id") id: Int,
        @Header("Authorization") token: String
    )

    // Product
    @POST("product")
    suspend fun insertProduct(
        @Body product: ProductRemote,
        @Header("Authorization") token: String
    ) : ProductRemote

    @PUT("product")
    suspend fun updateProduct(
        @Body product: ProductRemote,
        @Header("Authorization") token: String
    ): ProductRemote

    @DELETE("product/{id}")
    suspend fun deleteProduct(
        @Path("id") id: Int,
        @Header("Authorization") token: String
    )

    @GET("product/{id}")
    suspend fun getProductById(
        @Path("id") id: Int,
        @Header("Authorization") token: String
    ): ProductRemote

    @GET("products/{category_id}")
    suspend fun getProductsByCategoryId(
        @Path("category_id") category_id: Int,
        @Header("Authorization") token: String
    ): List<ProductRemote>

    // Role
    @POST("role")
    suspend fun getRole(
        @Body role: RoleRemote,
    ) : RoleRemote?

    @GET("roles")
    suspend fun getRoles() : List<RoleRemote>

    // Category
    @POST("category")
    suspend fun getCategory(
        @Body category: CategoryRemote,
    ) : CategoryRemote?

    @GET("categories")
    suspend fun getCategories() : List<CategoryRemote>

    @GET("categories/{after}")
    suspend fun getCategoriesList(
        @Path("after") after: Int,
        @Header("Authorization") token: String
    ) : CategoryListResponse

    // Status
    @POST("status")
    suspend fun getStatus(
        @Body status: StatusRemote,
    ) : StatusRemote?

    // Order
    @GET("order/cart")
    suspend fun getCartByUser(
        @Header("Authorization") token: String
    ) : OrderRemote

    @GET("order/to_worker/{after}")
    suspend fun getOrdersToWorkList(
        @Path("after") after: Int,
        @Header("Authorization") token: String
    ) : OrderListResponse

    @GET("order/to_user/{after}")
    suspend fun getOrdersByUserList(
        @Path("after") after: Int,
        @Header("Authorization") token: String
    ) : OrderListResponse

    @POST("order")
    suspend fun insertOrder(
        @Body order: OrderRemote,
        @Header("Authorization") token: String
    ) : OrderRemote?

    @PUT("order")
    suspend fun updateOrder(
        @Body order: OrderRemote,
        @Header("Authorization") token: String
    ) : OrderRemote?

    @DELETE("order/{id}")
    suspend fun deleteOrder(
        @Path("id") id: Int,
        @Header("Authorization") token: String
    )

    // Order_Product
    @POST("order/product")
    suspend fun insertOrderProduct(
        @Body orderProduct: OrderProductRemote,
        @Header("Authorization") token: String
    )

    @PUT("order/product")
    suspend fun updateOrderProduct(
        @Body orderProduct: OrderProductRemote,
        @Header("Authorization") token: String
    )

    @POST("order/product/delete")
    suspend fun deleteOrderProduct(
        @Body orderProduct: OrderProductRemote,
        @Header("Authorization") token: String
    )

    companion object {
        private const val BASE_URL ="http://10.0.2.2:8000/api/"

        @Volatile
        private var INSTANCE: MyServerService? = null

        fun getInstance(): MyServerService {
            return INSTANCE ?: synchronized(this) {
                val logger = HttpLoggingInterceptor()
                logger.level = HttpLoggingInterceptor.Level.BASIC
                val client = UnsafeOkHttpClient.getUnsafeOkHttpClient()
                return Retrofit.Builder()
                    .baseUrl(BASE_URL)
                    .client(client)
                    .addConverterFactory(Json.asConverterFactory("application/json".toMediaType()))
                    .build()
                    .create(MyServerService::class.java)
                    .also { INSTANCE = it }
            }
        }
    }
}