package com.example.mobileapp.data.api.mediators

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.models.CategoryListResponse
import com.example.mobileapp.data.api.models.toCategory
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.Category
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class CategoryRemoteMediator (
    private val database: AppDatabase,
    private val serverService: MyServerService,
    private val token: String
) : RemoteMediator<Int, Category>(){

    private val categoryDao = database.categoryDao()

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, Category>
    ): MediatorResult {
        return try {
            var loadKey = when (loadType) {
                LoadType.REFRESH -> null
                LoadType.PREPEND -> return MediatorResult.Success(endOfPaginationReached = true)
                LoadType.APPEND -> {
                    val lastItem = state.lastItemOrNull()
                        ?: return MediatorResult.Success(endOfPaginationReached = true)
                    lastItem.id
                }
            }

            if (loadKey == null) {
                loadKey = 0
            }
            val response: CategoryListResponse = serverService.getCategoriesList(after = loadKey, token = token)

            database.withTransaction {

                if (loadType == LoadType.REFRESH) {
                    categoryDao.deleteAll()
                }

                for (category in response.categories) {
                    categoryDao.insert(category.toCategory())
                }
            }

            MediatorResult.Success(endOfPaginationReached = response.nextKey == -1)
        } catch (e: IOException) {
            MediatorResult.Error(e)
        }
    }
}