package com.example.mobileapp.data.api.mediators

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.models.OrderListResponse
import com.example.mobileapp.data.api.models.toOrder
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.OrderWithProducts
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class OrderRemoteMediator (
    private val database: AppDatabase,
    private val serverService: MyServerService,
    private val query: String,
    private val token: String
) : RemoteMediator<Int, OrderWithProducts>(){

    private val orderDao = database.orderDao()

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, OrderWithProducts>
    ): MediatorResult {
        return try {
            var loadKey = when (loadType) {
                LoadType.REFRESH -> null
                LoadType.PREPEND -> return MediatorResult.Success(endOfPaginationReached = true)
                LoadType.APPEND -> {
                    val lastItem = state.lastItemOrNull()
                        ?: return MediatorResult.Success(endOfPaginationReached = true)
                    lastItem.order.id
                }
            }

            if (loadKey == null) {
                loadKey = 0
            }

            val response: OrderListResponse = if (query.isEmpty()) {
                serverService.getOrdersToWorkList (after = loadKey, token = token)
            } else {serverService.getOrdersByUserList(after = loadKey, token = token)}

            database.withTransaction {
                if (loadType == LoadType.REFRESH) {
                    orderDao.deleteByQuery(query)
                }

                for (order in response.orders) {
                    orderDao.insert(order.toOrder())
                }
            }

            MediatorResult.Success(endOfPaginationReached = response.nextKey == -1)
        } catch (e: IOException) {
            MediatorResult.Error(e)
        }
    }
}