package com.example.mobileapp.data.api.mediators

import androidx.paging.ExperimentalPagingApi
import androidx.paging.LoadType
import androidx.paging.PagingState
import androidx.paging.RemoteMediator
import androidx.room.withTransaction
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.models.UserListResponse
import com.example.mobileapp.data.api.models.toUser
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.User
import java.io.IOException

@OptIn(ExperimentalPagingApi::class)
class UserRemoteMediator (
    private val database: AppDatabase,
    private val serverService: MyServerService,
    private val token: String
) : RemoteMediator<Int, User>(){

    private val userDao = database.userDao()

    override suspend fun load(
        loadType: LoadType,
        state: PagingState<Int, User>
    ): MediatorResult {
        return try {
            var loadKey = when (loadType) {
                LoadType.REFRESH -> null
                LoadType.PREPEND -> return MediatorResult.Success(endOfPaginationReached = true)
                LoadType.APPEND -> {
                    val lastItem = state.lastItemOrNull()
                        ?: return MediatorResult.Success(endOfPaginationReached = true)
                    lastItem.id
                }
            }

            if (loadKey == null) {
                loadKey = 0
            }
            val response: UserListResponse = serverService.getUsersList(after = loadKey, token = token)

            database.withTransaction {

                if (loadType == LoadType.REFRESH) {
                    userDao.deleteAll()
                }

                for (user in response.users) {
                    userDao.insert(user.toUser())
                }
            }

            MediatorResult.Success(endOfPaginationReached = response.nextKey == -1)
        } catch (e: IOException) {
            MediatorResult.Error(e)
        }
    }
}