package com.example.mobileapp.data.db.dao
import androidx.paging.PagingSource
import androidx.room.*
import com.example.mobileapp.data.db.models.Order
import com.example.mobileapp.data.db.models.OrderWithProducts
import kotlinx.coroutines.flow.Flow

@Dao
interface OrderDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insert(order: Order)
    @Update
    suspend fun update(order: Order)
    @Delete
    suspend fun delete(order: Order)

    @Query("select * from `order` where `order`.`id` =:id")
    fun getById(id: Int): Flow<OrderWithProducts>

    @Query("select * from `order` where `order`.`user_id` =:userId and `order`.`status_id` = 1")
    fun getCartByUser(userId: Int): Flow<OrderWithProducts?>

    @Query("select * from `order` where `order`.`status_id` != 1 and `order`.`status_id` != 4")
    fun getOrdersToWork(): PagingSource<Int, OrderWithProducts>
    @Query("select * from `order` where (`order`.`user_id` =:userId and `order`.`status_id` != 1 and `order`.`status_id` != 4)")
    fun getOrdersByUser(userId: Int): PagingSource<Int, OrderWithProducts>

    @Query("delete from `order` where `order`.`status_id` != 1 and `order`.`status_id` != 4")
    fun deleteOrdersToWork()
    @Query("delete from `order` where (`order`.`user_id` =:userId and `order`.`status_id` != 1 and `order`.`status_id` != 4)")
    fun deleteOrdersByUser(userId: Int)

    fun deleteByQuery(query: String) {
        if (query.isEmpty()) {
            deleteOrdersToWork()
        }
        else {
            deleteOrdersByUser(query.toInt())
        }
    }
}