package com.example.mobileapp.data.repositories

import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.mediators.CategoryRemoteMediator
import com.example.mobileapp.data.api.models.CategoryRemote
import com.example.mobileapp.data.api.repositories.RestCategoryRepository
import com.example.mobileapp.data.db.dao.CategoryDao
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.Category
import kotlinx.coroutines.flow.Flow
import javax.inject.Inject

class CategoryRepository @Inject constructor(
    private val categoryDao: CategoryDao,
    private val database: AppDatabase,
    private val restRepository: RestCategoryRepository
){
    @OptIn(ExperimentalPagingApi::class)
    fun getAllPaged(token: String): Flow<PagingData<Category>> = Pager(
        config = PagingConfig(
            pageSize = 20,
            enablePlaceholders = false
        ),
        pagingSourceFactory = categoryDao::getPaged,
        remoteMediator = CategoryRemoteMediator(database = database, serverService = MyServerService.getInstance(), token = token)
    ).flow

    suspend fun getCategories() : List<CategoryRemote> {
        return restRepository.getCategories()
    }
}