package com.example.mobileapp.data.repositories

import com.example.mobileapp.data.db.dao.OrderDao
import androidx.paging.*
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.models.toOrder
import com.example.mobileapp.data.api.models.toOrderProduct
import com.example.mobileapp.data.api.models.toOrderRemote
import com.example.mobileapp.data.api.repositories.RestOrderRepository
import com.example.mobileapp.data.db.dao.OrderProductDao
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.Order
import com.example.mobileapp.data.api.mediators.OrderRemoteMediator
import com.example.mobileapp.data.db.models.OrderWithProducts
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import javax.inject.Inject

class OrderRepository @Inject constructor(
    private val database: AppDatabase,
    private val orderDao: OrderDao,
    private val orderProductDao: OrderProductDao,
    private val restRepository: RestOrderRepository
){
    suspend fun insert(token: String, order: Order): Order? {
        orderDao.insert(order)
        val result =  restRepository.insertOrder(token, order.toOrderRemote())
        if (result == null) {
            return result
        }
        return result.toOrder()
    }
    suspend fun update(token: String, order: Order) {
        orderDao.update(order)
        restRepository.updateOrder(token, order.toOrderRemote())
    }
    suspend fun delete(token: String, order: Order) {
        orderProductDao.deleteByOrderId(order.id!!)
        orderDao.delete(order)
        restRepository.deleteOrder(token, order.id)
    }
    fun getById(id: Int) = orderDao.getById(id)

    suspend fun getCartByUser(token: String) : OrderWithProducts? {
        val order = restRepository.getCartByUser(token)
        if (order.id != null) {
            orderDao.insert(order.toOrder())
            for (product in order.order_products) {
                orderProductDao.insert(product.toOrderProduct())
            }
            return orderDao.getCartByUser(order.user_id!!).first()
        }
        return null
    }

    @OptIn(ExperimentalPagingApi::class)
    fun getOrdersToWorkPaged(token: String): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = orderDao::getOrdersToWork,
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = "", token = token)
    ).flow

    @OptIn(ExperimentalPagingApi::class)
    fun getOrdersByUserPaged(token: String, userId: Int): Flow<PagingData<OrderWithProducts>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = {orderDao.getOrdersByUser(userId)},
        remoteMediator = OrderRemoteMediator(database = database, serverService = MyServerService.getInstance(), query = userId.toString(), token = token)
    ).flow
}