package com.example.mobileapp.data.repositories

import com.example.mobileapp.data.api.models.ProductRemote
import com.example.mobileapp.data.api.models.toProduct
import com.example.mobileapp.data.api.repositories.RestProductRepository
import com.example.mobileapp.data.db.dao.OrderProductDao
import com.example.mobileapp.data.db.dao.ProductDao
import com.example.mobileapp.data.db.models.Product
import javax.inject.Inject

class ProductRepository @Inject constructor(
    private val productDao: ProductDao,
    private val orderProductDao: OrderProductDao,
    private val restRepository: RestProductRepository
){
    suspend fun insert(product: Product, token: String) {
        productDao.insert(product)
        return restRepository.insert(product, token)
    }
    suspend fun update(product: Product, token: String) {
        productDao.update(product)
        return restRepository.update(product, token)
    }
    suspend fun delete(product: Product, token: String) {
        productDao.delete(product)
        product.id?.let { orderProductDao.deleteByProductId(it) }
        return restRepository.delete(product.id!!, token)
    }

    suspend fun getProductById(id: Int, token: String) : Product {
        val product = restRepository.getProductById(id, token)
        productDao.insert(product)
        return product
    }

    suspend fun getProductsByCategoryId(categoryId: Int, token: String) : List<ProductRemote> {
        val products = restRepository.getProductsByCategoryId(categoryId, token)
        products.forEach{ productRemote ->
            productDao.insert(productRemote.toProduct())
        }
        return products
    }
}