package com.example.mobileapp.data.repositories

import androidx.paging.ExperimentalPagingApi
import androidx.paging.Pager
import androidx.paging.PagingConfig
import androidx.paging.PagingData
import com.example.mobileapp.data.api.MyServerService
import com.example.mobileapp.data.api.mediators.UserRemoteMediator
import com.example.mobileapp.data.api.models.TokenRemote
import com.example.mobileapp.data.api.models.UserRemote
import com.example.mobileapp.data.api.repositories.RestUserRepository
import com.example.mobileapp.data.db.dao.UserDao
import com.example.mobileapp.data.db.AppDatabase
import com.example.mobileapp.data.db.models.User
import kotlinx.coroutines.flow.Flow
import javax.inject.Inject

class UserRepository @Inject constructor(
    private val userDao: UserDao,
    private val database: AppDatabase,
    private val restRepository: RestUserRepository
){
    suspend fun insert(user: User){
        userDao.insert(user)
        restRepository.insert(user)
    }
    suspend fun update(user: User, token: String) {
        return restRepository.update(user, token)
    }
    suspend fun delete(user: User, token: String) {
        userDao.delete(user)
        return restRepository.delete(user.id!!, token)
    }
    @OptIn(ExperimentalPagingApi::class)
    fun getAllPaged(token: String): Flow<PagingData<User>> = Pager(
        config = PagingConfig(
            pageSize = 6,
            enablePlaceholders = false
        ),
        pagingSourceFactory = userDao::getPaged,
        remoteMediator = UserRemoteMediator(database = database, serverService = MyServerService.getInstance(), token = token)
    ).flow

    suspend fun getToken(user: User): TokenRemote {
        return restRepository.getToken(user)
    }

    suspend fun checkLogin(user: User): UserRemote? {
        return restRepository.checkLogin(user)
    }
}