package com.example.mobileapp.ui.screens

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.layout.size
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedButton
import androidx.compose.material3.Text
import androidx.compose.runtime.Composable
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.res.painterResource
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.navigation.NavController
import com.example.mobileapp.R
import com.example.mobileapp.ui.navigation.Screens
import com.example.mobileapp.ui.widgets.Header

@Composable
fun ManagmentScreen(navController: NavController){
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color(212, 206, 203))
        ManagmentCard(navController)
    }
}

@Composable
fun ManagmentCard(navController: NavController){
    Column(
        modifier = Modifier
            .fillMaxSize()
            .background(Color(212, 206, 203)),
        horizontalAlignment = Alignment.CenterHorizontally,

    ) {
        Column(
            modifier = Modifier
                .fillMaxHeight()
                .fillMaxWidth(0.9f)
                .background(Color.White),
            horizontalAlignment = Alignment.CenterHorizontally,
        ) {
            Text(
                text = "Блюда",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )
            Row(
                horizontalArrangement = Arrangement.spacedBy(5.dp),
                modifier = Modifier.padding(top = 10.dp),
            ){
                OutlinedButton(
                    onClick = {
                        navController.navigate(Screens.CreateProductScreen.name)
                    },
                    border = BorderStroke(4.dp, Color(222,161,69)),
                    shape = RoundedCornerShape(20.dp),
                    colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),

                    ){
                    Icon(
                        painterResource(id = R.drawable.icon_add),
                        contentDescription = null,
                        Modifier.size(50.dp),
                        tint = Color.Black
                    )
                }

                OutlinedButton(
                    onClick = {
                        navController.navigate(Screens.EditMenuScreen.name)
                    },
                    border = BorderStroke(4.dp, Color(222,161,69)),
                    shape = RoundedCornerShape(20.dp),
                    colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),

                    ){
                    Icon(
                        painterResource(id = R.drawable.icon_settings),
                        contentDescription = null,
                        Modifier.size(50.dp),
                        tint = Color.Black
                    )
                }
            }
            Text(
                text = "Пользователи",
                color = Color.Black,
                fontFamily = FontFamily(Font(R.font.nunito_extrabold_italic)),
                fontSize = 34.sp,
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(top = 10.dp),
                textAlign = TextAlign.Center,
            )
            OutlinedButton(
                onClick = {
                    navController.navigate(Screens.UsersScreen.name)
                },
                border = BorderStroke(4.dp, Color(222,161,69)),
                shape = RoundedCornerShape(20.dp),
                colors = ButtonDefaults.outlinedButtonColors(Color(255,186,83)),
                modifier = Modifier.padding(top = 10.dp)
                ){
                Icon(
                    painterResource(id = R.drawable.icon_profile),
                    contentDescription = null,
                    Modifier.size(50.dp),
                    tint = Color.Black
                )
                Icon(
                    painterResource(id = R.drawable.icon_order),
                    contentDescription = null,
                    Modifier.size(50.dp),
                    tint = Color.Black
                )
                Icon(
                    painterResource(id = R.drawable.icon_managment),
                    contentDescription = null,
                    Modifier.size(50.dp),
                    tint = Color.Black
                )
            }
        }
    }
}