package com.example.mobileapp.ui.screens

import androidx.compose.foundation.BorderStroke
import androidx.compose.foundation.background
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.shape.RoundedCornerShape
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.Button
import androidx.compose.material3.ButtonDefaults
import androidx.compose.material3.Card
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.material3.TextFieldDefaults
import androidx.compose.runtime.Composable
import androidx.compose.runtime.livedata.observeAsState
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.graphics.Color
import androidx.compose.ui.text.TextStyle
import androidx.compose.ui.text.font.Font
import androidx.compose.ui.text.font.FontFamily
import androidx.compose.ui.text.font.FontWeight
import androidx.compose.ui.text.input.TextFieldValue
import androidx.compose.ui.text.style.TextAlign
import androidx.compose.ui.unit.dp
import androidx.compose.ui.unit.sp
import androidx.hilt.navigation.compose.hiltViewModel
import androidx.navigation.NavController
import com.example.mobileapp.R
import com.example.mobileapp.ui.navigation.Screens
import com.example.mobileapp.viewmodels.RegistrationViewModel
import com.example.mobileapp.ui.widgets.Header

@Composable
fun RegistrationScreen(navController: NavController) {
    Column (
        modifier = Modifier.fillMaxSize(),
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Header(Color.White)
        RegistrationCard(navController)
    }
}

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun RegistrationCard(navController: NavController){
    val login = remember { mutableStateOf(TextFieldValue("")) }
    val password = remember { mutableStateOf(TextFieldValue("")) }
    val passwordRepeat = remember { mutableStateOf(TextFieldValue("")) }
    val viewModel = hiltViewModel<RegistrationViewModel>()
    val success = viewModel.successState.observeAsState().value

    if (success == true) {
        navController.navigate(Screens.AuthorizationScreen.name)
    }

    if (success == false) {
        AlertDialog(
            title = {
                Text(
                    text = "Ошибка регистрации",
                    fontWeight = FontWeight.Bold,
                    fontSize = 20.sp
                )
            },
            text = {
                Text(
                    text = "Проверьте корректность введенных данных",
                    fontWeight = FontWeight.Normal,
                    fontSize = 16.sp
                )
            },
            onDismissRequest = {
                viewModel.calmSuccessState()
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        viewModel.calmSuccessState()
                    }
                ) {
                    Text("ОК")
                }
            }
        )
    }

    Column(
        horizontalAlignment = Alignment.CenterHorizontally,
    ) {
        Card(
            shape = RoundedCornerShape(30.dp),
            modifier = Modifier.padding(top = 30.dp)
        ) {

            Column (
                horizontalAlignment = Alignment.CenterHorizontally,
                modifier = Modifier
                    .background(Color(212, 206, 203))
                    .fillMaxWidth(0.9f)
            ) {
                Card(
                    shape = RoundedCornerShape(90.dp, 90.dp, 30.dp, 30.dp),
                    border = BorderStroke(4.dp, Color(222,161,69)),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 30.dp, start = 20.dp, end = 20.dp)
                ){
                    Text(
                        text = "Регистрация",
                        color = Color.White,
                        fontFamily = FontFamily(Font(R.font.nunito_bold)),
                        fontSize = 30.sp,
                        modifier = Modifier
                            .fillMaxWidth()
                            .background(Color(255, 186, 83))
                            .padding(top = 20.dp, bottom = 20.dp),
                        textAlign = TextAlign.Center,
                    )
                }

                OutlinedTextField(
                    value = login.value,
                    onValueChange = {login.value = it},
                    placeholder = { Text(text = "Логин", fontSize = 20.sp, color = Color.LightGray) },
                    singleLine = true,

                    shape = RoundedCornerShape(30.dp, 30.dp, 30.dp, 30.dp),
                    textStyle = TextStyle(fontSize = 20.sp),
                    colors = TextFieldDefaults.outlinedTextFieldColors(
                        textColor = Color.Black,
                        disabledTextColor = Color.Transparent,
                        containerColor = Color.White,
                        focusedBorderColor = Color.Yellow,
                        unfocusedBorderColor = Color.Yellow,
                    ),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 20.dp, start = 20.dp, end = 20.dp),
                )

                OutlinedTextField(
                    value = password.value,
                    onValueChange = {password.value = it},
                    placeholder = { Text(text = "Пароль", fontSize = 20.sp, color = Color.LightGray) },
                    singleLine = true,

                    shape = RoundedCornerShape(30.dp, 30.dp, 30.dp, 30.dp),
                    textStyle = TextStyle(fontSize = 20.sp),
                    colors = TextFieldDefaults.outlinedTextFieldColors(
                        textColor = Color.Black,
                        disabledTextColor = Color.Transparent,
                        containerColor = Color.White,
                        focusedBorderColor = Color(183,97,48),
                        unfocusedBorderColor = Color(183,97,48),
                    ),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 20.dp, start = 20.dp, end = 20.dp),
                )

                OutlinedTextField(
                    value = passwordRepeat.value,
                    onValueChange = {passwordRepeat.value = it},
                    placeholder = { Text(text = "Логин повторно", fontSize = 20.sp, color = Color.LightGray) },
                    singleLine = true,

                    shape = RoundedCornerShape(30.dp, 30.dp, 30.dp, 30.dp),
                    textStyle = TextStyle(fontSize = 20.sp),
                    colors = TextFieldDefaults.outlinedTextFieldColors(
                        textColor = Color.Black,
                        disabledTextColor = Color.Transparent,
                        containerColor = Color.White,
                        focusedBorderColor = Color.Red,
                        unfocusedBorderColor = Color.Red,
                    ),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 20.dp, start = 20.dp, end = 20.dp),
                )

                Button(
                    onClick = {
                        viewModel.registration(login.value.text, password.value.text, passwordRepeat.value.text)
                    },
                    colors = ButtonDefaults.buttonColors(Color(255,186,83), Color.White),
                    shape = RoundedCornerShape(30.dp, 30.dp, 30.dp, 30.dp),
                    border = BorderStroke(4.dp, Color(222,161,69)),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 20.dp, start = 20.dp, end = 20.dp)
                ) {
                    Text(
                        text = "Продолжить",
                        color = Color.White,
                        fontFamily = FontFamily(Font(R.font.nunito_bold)),
                        fontSize = 30.sp,
                        textAlign = TextAlign.Center,
                    )
                }

                Button(
                    onClick = {
                        navController.navigate(Screens.AuthorizationScreen.name)
                    },
                    colors = ButtonDefaults.buttonColors(Color(212,206,203)),
                    modifier = Modifier
                        .fillMaxWidth()
                        .padding(top = 10.dp, start = 20.dp, end = 20.dp)
                ) {
                    Text(
                        text = "Есть аккаунт",
                        color = Color.White,
                        fontFamily = FontFamily(Font(R.font.nunito_bold)),
                        fontSize = 20.sp,
                        textAlign = TextAlign.Center,
                    )
                }
            }
        }
    }
}