package com.example.mobileapp.viewmodels

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.data.db.models.Role
import com.example.mobileapp.data.db.models.User
import com.example.mobileapp.data.repositories.RoleRepository
import com.example.mobileapp.data.repositories.UserRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class AuthorizationViewModel @Inject constructor(
    private val userRepository: UserRepository,
    private val roleRepository: RoleRepository
) : ViewModel() {
    private val _successState = MutableLiveData<Boolean?>()
    val successState: LiveData<Boolean?>
        get() = _successState

    fun calmSuccessState() {
        _successState.postValue(null)
    }

    fun login(sharedPref: PreferencesManager, login: String, password: String) {
        viewModelScope.launch {
            val token_response = userRepository.getToken(User(null, login, password,-1))
            if (token_response.access_token.isNotEmpty()) {
                val user = userRepository.checkLogin(User(null, login, password, -1))
                if (user != null){
                    val role = roleRepository.getRole(Role(user.role_id, ""))
                    if (role != null){
                        sharedPref.saveData("userId", user.id.toString())
                        sharedPref.saveData("userRole", role.name)
                        sharedPref.saveData("token", token_response.access_token)
                        _successState.postValue(true)
                        println(token_response.access_token)
                    }
                }
            }
            else {
                _successState.postValue(false)
            }
        }
    }
}