package com.example.mobileapp.viewmodels

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobileapp.data.db.models.OrderProduct
import com.example.mobileapp.data.db.models.OrderWithProducts
import com.example.mobileapp.data.repositories.OrderProductRepository
import com.example.mobileapp.data.repositories.OrderRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class CartViewModel @Inject constructor(
    private val orderRepository: OrderRepository,
    private val orderProductRepository: OrderProductRepository
) : ViewModel() {
    private val _cart = MutableLiveData<OrderWithProducts?>()
    val cart: LiveData<OrderWithProducts?>
        get() = _cart

    fun getCart(token: String) {
        viewModelScope.launch {
            _cart.postValue(orderRepository.getCartByUser(token))
        }
    }

    fun makeOrder(token: String) {
        if (cart.value == null){
            return
        }
        val model = cart.value!!.order
        model.statusId = 2
        viewModelScope.launch {
            orderRepository.update(token, model)
        }
        _cart.postValue(null)
    }

    fun removeProductFromOrder(token: String, productId: Int) {
        val orderModel = cart.value!!.order
        var productModel : OrderProduct? = null

        cart.value!!.orderWithProducts.map {
            if (it.orderProduct.productId == productId){
                productModel = it.orderProduct
                orderModel.price -= it.product.price
            }
        }

        if(productModel!!.amount == 1) {
            // delete
            viewModelScope.launch {
                orderRepository.update(token, orderModel)
                orderProductRepository.delete(token, productModel!!)
                getCart(token)
            }
        }
        else{
            // update
            productModel!!.amount -= 1
            viewModelScope.launch {
                orderRepository.update(token, orderModel)
                orderProductRepository.update(token, productModel!!)
                getCart(token)
            }
        }
    }
    fun addProductToOrder(token: String, productId: Int) {
        val orderModel = cart.value!!.order
        var productModel : OrderProduct? = null
        cart.value!!.orderWithProducts.map {
            if (it.orderProduct.productId == productId){
                productModel = it.orderProduct
                orderModel.price += it.product.price
            }
        }
        // update
        productModel!!.amount += 1
        viewModelScope.launch {
            orderRepository.update(token, orderModel)
            orderProductRepository.update(token, productModel!!)
            getCart(token)
        }

    }
}