package com.example.mobileapp.viewmodels

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.PagingData
import com.example.mobileapp.data.api.models.ProductRemote
import com.example.mobileapp.data.api.models.toProduct
import com.example.mobileapp.data.db.models.Category
import com.example.mobileapp.data.db.models.Order
import com.example.mobileapp.data.db.models.OrderProduct
import com.example.mobileapp.data.db.models.Product
import com.example.mobileapp.data.repositories.CategoryRepository
import com.example.mobileapp.data.repositories.OrderProductRepository
import com.example.mobileapp.data.repositories.OrderRepository
import com.example.mobileapp.data.repositories.ProductRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import java.util.stream.Collectors
import javax.inject.Inject

@HiltViewModel
class MenuViewModel @Inject constructor(
    private val categoryRepository: CategoryRepository,
    private val orderRepository: OrderRepository,
    private val orderProductRepository: OrderProductRepository,
    private val productRepository: ProductRepository
) : ViewModel() {
    fun getCategoriesList(token: String) : Flow<PagingData<Category>> {
        return categoryRepository.getAllPaged(token)
    }

    fun getProductsByCategoryId(categoryId: Int, token: String): Flow<MutableList<Product>?> {
        return flow{ emit(productRepository.getProductsByCategoryId(categoryId, token).stream().map(
            ProductRemote::toProduct).collect(Collectors.toList())) }
    }

    fun deleteProduct(product : Product, token: String) {
        viewModelScope.launch {
            productRepository.delete(product, token)
        }
    }

    fun addProductToCart(productId: Int, userId: Int, token: String) {
        viewModelScope.launch {
            val product = productRepository.getProductById(productId, token)
            val order = orderRepository.getCartByUser(token)

            if (order == null) {
                val newOrder = orderRepository.insert(token, Order(null, userId, product.price, 1 ))
                if (newOrder != null) {
                    orderProductRepository.insert(token, OrderProduct(newOrder.id!!, productId, 1))
                }
            }
            else {
                var isAlreadyAdded = false
                for (prod in order.orderWithProducts) {
                    if (prod.product != null) {
                        if (prod.product.id == productId) {
                            val newOrderProduct = prod.orderProduct
                            newOrderProduct.amount += 1
                            val newOrder = order.order
                            newOrder.price += prod.product.price
                            orderProductRepository.update(token, newOrderProduct)
                            orderRepository.update(token, newOrder)
                            isAlreadyAdded = true
                        }
                    }
                }
                if (!isAlreadyAdded) {
                    orderProductRepository.insert(token, OrderProduct(order.order.id!!, productId, 1))
                    val newOrder = order.order
                    newOrder.price += product.price
                    orderRepository.update(token, newOrder)
                }
            }
        }
    }
}