package com.example.mobileapp.viewmodels

import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.PagingData
import com.example.mobileapp.data.db.models.OrderWithProducts
import com.example.mobileapp.data.db.models.Status
import com.example.mobileapp.data.repositories.OrderRepository
import com.example.mobileapp.data.repositories.StatusRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.first
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class OrderViewModel @Inject constructor(
    private val statusRepository: StatusRepository,
    private val orderRepository: OrderRepository
) : ViewModel() {

    fun getOrdersList(token: String) : Flow<PagingData<OrderWithProducts>> {
        return orderRepository.getOrdersToWorkPaged(token)
    }

    suspend fun getOrderStatus(statusId: Int) : String{
        return statusRepository.getStatus(Status(statusId, ""))!!.name
    }

    fun changeOrderStatus(token: String, orderId: Int) {
        viewModelScope.launch {
            val order = orderRepository.getById(orderId).first()

            when(statusRepository.getStatus(Status(order.order.statusId, ""))!!.name){
                "Принят" -> {
                    val newOrder = order.order
                    newOrder.statusId = statusRepository.getStatus(Status(null, "Готов"))!!.id!!
                    orderRepository.update(token, newOrder)
                }
                "Готов" -> {
                    val newOrder = order.order
                    newOrder.statusId = statusRepository.getStatus(Status(null, "Выдан"))!!.id!!
                    orderRepository.update(token, newOrder)
                }
            }
        }
    }
}