package com.example.mobileapp.viewmodels

import androidx.lifecycle.ViewModel
import androidx.paging.PagingData
import com.example.mobileapp.data.sharedpref.PreferencesManager
import com.example.mobileapp.data.db.models.OrderWithProducts
import com.example.mobileapp.data.db.models.Status
import com.example.mobileapp.data.repositories.OrderRepository
import com.example.mobileapp.data.repositories.StatusRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.Flow
import javax.inject.Inject

@HiltViewModel
class ProfileViewModel @Inject constructor(
    private val orderRepository: OrderRepository,
    private val statusRepository: StatusRepository
) : ViewModel() {
    fun getActiveOrders(token: String, userId: Int) : Flow<PagingData<OrderWithProducts>>? {
        if (token.isNotEmpty() && userId.toString().isNotEmpty()){
            return orderRepository.getOrdersByUserPaged(token, userId)
        }
        return null
    }

    suspend fun getOrderStatus(statusId: Int) : String{
        return statusRepository.getStatus(Status(statusId, ""))!!.name
    }

    fun logout(sharedPref: PreferencesManager) {
        sharedPref.deleteData("userId")
        sharedPref.deleteData("userRole")
        sharedPref.deleteData("token")
    }
}