package com.example.mobileapp.viewmodels

import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.mobileapp.data.db.models.User
import com.example.mobileapp.data.repositories.UserRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.launch
import javax.inject.Inject

@HiltViewModel
class RegistrationViewModel @Inject constructor(
    private val userRepository: UserRepository
) : ViewModel() {
    private val _successState = MutableLiveData<Boolean?>()
    val successState: LiveData<Boolean?>
        get() = _successState

    fun calmSuccessState() {
        _successState.postValue(null)
    }

    fun registration(login: String, password: String, confirmPassword: String) {
        if (login.length < 3 || login.length > 20) {
            _successState.postValue(false)
            return
        }

        if (password.length < 3 || password.length > 20) {
            _successState.postValue(false)
            return
        }

        if (password != confirmPassword) {
            _successState.postValue(false)
            return
        }

        viewModelScope.launch {
            val user_response = userRepository.checkLogin(User(null, login, password, -1))
            if (user_response != null) {
                if (user_response.login.isNullOrEmpty()) {
                    if (login == "admin" && password == "admin") {
                        userRepository.insert(User(null, login, password, 1))
                        _successState.postValue(true)
                    }
                    else {
                        userRepository.insert(User(null, login, password, 3))
                        _successState.postValue(true)
                    }
                }
                else {
                    _successState.postValue(false)
                }
            }
        }
    }
}