package com.example.mobileapp.viewmodels

import androidx.compose.runtime.mutableStateListOf
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import androidx.paging.PagingData
import com.example.mobileapp.data.api.models.RoleRemote
import com.example.mobileapp.data.api.models.toRole
import com.example.mobileapp.data.db.models.Role
import com.example.mobileapp.data.db.models.User
import com.example.mobileapp.data.repositories.RoleRepository
import com.example.mobileapp.data.repositories.UserRepository
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.launch
import java.util.stream.Collectors
import javax.inject.Inject

@HiltViewModel
class UsersViewModel @Inject constructor(
    private val userRepository: UserRepository,
    private val roleRepository: RoleRepository
) : ViewModel() {
    private val _allRoles = mutableStateListOf<Role>()
    val allRoles: List<Role>
        get() = _allRoles

    init {
        viewModelScope.launch {
            _allRoles.addAll(roleRepository.getRoles().stream().map(RoleRemote::toRole).collect(Collectors.toList()))
        }
    }

    fun getUsersList(token: String) : Flow<PagingData<User>> {
        return userRepository.getAllPaged(token)
    }

    fun getUserRole(roleId: Int) : String {
        return allRoles.find { it.id == roleId }?.name ?: ""
    }

    fun changeUserRole(userId: Int, roleName: String, token: String) {
        viewModelScope.launch {
            userRepository.update(
                User(
                    userId, "","",
                    allRoles.find { it.name == roleName }!!.id!!),
                token
            )
        }
    }

    fun deleteUser(userId: Int, token: String){
        viewModelScope.launch{
            userRepository.delete(User(userId, "","",-1),token)
        }
    }
}